#  AdaBoost classifier hyper parameter tunning
import pandas as pd
from sklearn.ensemble import AdaBoostClassifier
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics import accuracy_score
from sklearn.metrics import make_scorer, confusion_matrix
from sklearn.model_selection import train_test_split, RandomizedSearchCV, KFold
from sklearn.pipeline import Pipeline
from preprocessor import pre_process_data
#from preprocessor import get_types

SAVE_MODEL = True
MODELS_DIR = "models"
DATA_DIR = "data"
DIMENSIONS = ["IE", "NS", "FT", "PJ"]

data = pd.read_csv('data/mbti_personality.csv');
#data = data.join(data.apply(lambda row: get_types(row), axis=1))

type_indicators = ["IE: Introversion (I) - Extroversion (E)", "NS: Intuition (N) â Sensing (S)",
                   "FT: Feeling (F) - Thinking (T)", "JP: Judging (J) â Perceiving (P)"]

list_posts, list_personality = pre_process_data(data, remove_stop_words=True)

# Learn the vocabulary dictionary and return term-document matrix
Tfidf = TfidfVectorizer(analyzer="word",
                        max_features=1500,
                        tokenizer=None,
                        preprocessor=None,
                        stop_words=None,
                        max_df=0.7,
                        min_df=0.1)

X = list_posts

# train type indicator individually
for l in range(len(DIMENSIONS)):
    print("%s ..." % (type_indicators[l]))

    # Let's train type indicator individually
    Y = list_personality[:, l]

    # split data into train and test sets
    seed = 7
    test_size = 0.33
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=test_size, random_state=seed)

    # to improve the performance , use support vector classifier as base_estimator
    # svc = SVC(probability=True, kernel='linear')

    # Create adaboost classifer object with default parameters
    # for Adaboost the base_estimator = decision tree id default
    # n_estimators -  number of models to iteratively train / num
    # learning_rate - its the contribution of each model to the weights
    abc = AdaBoostClassifier()
    pipe = Pipeline([("Tfidf", Tfidf), ("abc", abc)])

    # Define our search space for grid search
    search_space = [
        {
            'abc__n_estimators': [50, 100, 300, 500],
            'abc__learning_rate': [0.0001, 0.001, 0.01, 0.1, 1.0]

        }
    ]

    # Define cross validation
    kfold = KFold(n_splits=10, random_state=42, shuffle=True)  # put n_splits =  10 , put shuffle = true,

    # AUC and accuracy as score
    scoring = {'AUC': 'roc_auc', 'Accuracy': make_scorer(accuracy_score)}

    # Define  Randomized Search
    grid = RandomizedSearchCV(
        pipe,
        param_distributions=search_space,
        cv=kfold,
        scoring=scoring,
        refit='AUC',
        verbose=1,
        n_jobs=-1
    )

    # for param in grid.get_params().keys():
    #     print(param)
    # Fit grid search
    model = grid.fit(X_train, y_train)

    # The model scores and confusion matrix can be obtained by
    predict = model.predict(X_test)
    print('Best AUC Score: {}'.format(model.best_score_))
    print('Accuracy: {}'.format(accuracy_score(y_test, predict)))
    print(confusion_matrix(y_test, predict))

    # print the  the best parameters
    print(model.best_params_)
