# Train and test hte dataset on XGBoost classifier
import os
import re
import numpy as np
import pandas as pd
from future.moves import pickle
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics import accuracy_score
from sklearn.model_selection import train_test_split
from sklearn.pipeline import Pipeline
from xgboost import XGBClassifier
from preprocessor import pre_process_data
from preprocessor import translate_personality
from preprocessor import translate_back
#from preprocessor import get_types

SAVE_MODEL = True
MODELS_DIR = "models"
DATA_DIR = "data"
DIMENSIONS = ["IE", "NS", "FT", "PJ"]

data = pd.read_csv('data/mbti_personality.csv')

list_posts, list_personality = pre_process_data(data, remove_stop_words=True)

# Learn the vocabulary dictionary and return term-document matrix
print("CountVectorizer...")

Tfidf = TfidfVectorizer(analyzer="word",
                        max_features=1500,
                        tokenizer=None,
                        preprocessor=None,
                        stop_words=None,
                        max_df=0.7,
                        min_df=0.1)

type_indicators = ["IE: Introversion (I) - Extroversion (E)", "NS: Intuition (N) â Sensing (S)",
                   "FT: Feeling (F) - Thinking (T)", "JP: Judging (J) â Perceiving (P)"]
X = list_posts

# setup parameters for xgboost
param = {}
param['n_estimators'] = 150  # 200
param['max_depth'] = 3  # 2
#param['nthread'] = 8
param['learning_rate'] = 0.01
param['gamma'] = 0.1
# param['xgb__colsample_bytree'] = 0.1

# Let's train type indicator individually
for l in range(len(DIMENSIONS)):
    print("%s ..." % (type_indicators[l]))

    # Let's train type indicator individually
    Y = list_personality[:, l]

    # split data into train and test sets
    seed = 7
    test_size = 0.33
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=test_size, random_state=seed)

    xgb = XGBClassifier(**param)
    pipe = Pipeline([("Tfidf", Tfidf), ("xgb", xgb)])
    pipe.fit(X_train, y_train)

    with open  (os.path.join(MODELS_DIR, "xgb_pipeline_{}.pkl".format(DIMENSIONS[l])), 'wb') as picklefile:
        pickle.dump(pipe, picklefile)

    # make predictions for test data
    y_pred = pipe.predict(X_test)
    predictions = [round(value) for value in y_pred]
    # evaluate predictions
    accuracy = accuracy_score(y_test, predictions)
    print("* %s Accuracy: %.2f%%" % (type_indicators[l], accuracy * 100.0))
