# Code to preprocess the user's text posts
import re

import numpy as np
import pandas as pd
from nltk.corpus import stopwords
from nltk.stem import PorterStemmer, WordNetLemmatizer

SAVE_MODEL = True
MODELS_DIR = "models"
DATA_DIR = "data"
DIMENSIONS = ["IE", "NS", "FT", "PJ"]


# data = pd.read_csv('data/mbti_personality.csv')

# add 4  columns for personality type indicators
# def get_types(row):
#     t = row['type']
#
#     I = 0
#     N = 0
#     T = 0
#     J = 0
#
#     if t[0] == 'I':
#         I = 1
#     elif t[0] == 'E':
#         I = 0
#     else:
#         print('I-E incorrect')
#
#     if t[1] == 'N':
#         N = 1
#     elif t[1] == 'S':
#         N = 0
#     else:
#         print('N-S incorrect')
#
#     if t[2] == 'T':
#         T = 1
#     elif t[2] == 'F':
#         T = 0
#     else:
#         print('T-F incorrect')
#
#     if t[3] == 'J':
#         J = 1
#     elif t[3] == 'P':
#         J = 0
#     else:
#         print('J-P incorrect')
#     return pd.Series({'IE': I, 'NS': N, 'TF': T, 'JP': J})


# data = data.join(data.apply(lambda row: get_types(row), axis=1))

# print("Introversion (I) /  Extroversion (E):\t", data['IE'].value_counts()[0], " / ", data['IE'].value_counts()[1])
# print("Intuition (N) â Sensing (S):\t\t", data['NS'].value_counts()[0], " / ", data['NS'].value_counts()[1])
# print("Thinking (T) â Feeling (F):\t\t", data['TF'].value_counts()[0], " / ", data['TF'].value_counts()[1])
# print("Judging (J) â Perceiving (P):\t\t", data['JP'].value_counts()[0], " / ", data['JP'].value_counts()[1])

b_Pers = {'I': 0, 'E': 1, 'N': 0, 'S': 1, 'F': 0, 'T': 1, 'J': 0, 'P': 1}
b_Pers_list = [{0: 'I', 1: 'E'}, {0: 'N', 1: 'S'}, {0: 'F', 1: 'T'}, {0: 'J', 1: 'P'}]


def translate_personality(personality):
    # transform mbti to binary vector

    return [b_Pers[l] for l in personality]


def translate_back(personality):
    # transform binary vector to mbti personality

    s = ""
    for i, l in enumerate(personality):
        s += b_Pers_list[i][l]
    return s


# To remove the personality type from the psosts
unique_type_list = ['INFJ', 'ENTP', 'INTP', 'INTJ', 'ENTJ', 'ENFJ', 'INFP', 'ENFP',
                    'ISFP', 'ISTP', 'ISFJ', 'ISTJ', 'ESTP', 'ESFP', 'ESTJ', 'ESFJ']

unique_type_list = [x.lower() for x in unique_type_list]

# Lemmatize
stemmer = PorterStemmer()
lemmatiser = WordNetLemmatizer()

# Cache the stop words for speed
cachedStopWords = stopwords.words("english")


def pre_process_data(data, remove_stop_words=True, remove_mbti_profiles=True):
    list_personality = []
    list_posts = []
    len_data = len(data)
    i = 0

    for row in data.iterrows():
        i += 1
        if i % 500 == 0 or i == 1 or i == len_data:
            print("%s of %s rows" % (i, len_data))

        # Remove and clean posts
        posts = row[1].posts
        # remove urls
        temp = re.sub('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', ' ', posts)
        # remove uncleared words
        temp = re.sub("[^a-zA-Z]", " ", temp)
        # Remove spaces > 1
        temp = re.sub(' +', ' ', temp).lower()

        # Remove stop words
        if remove_stop_words:
            temp = " ".join([lemmatiser.lemmatize(w) for w in temp.split(' ') if w not in cachedStopWords])
        else:
            temp = " ".join([lemmatiser.lemmatize(w) for w in temp.split(' ')])

        # Remove MBTI personality words from posts
        if remove_mbti_profiles:
            for t in unique_type_list:
                temp = temp.replace(t, "")

        # transform mbti to binary vector
        type_labelized = translate_personality(row[1].type)
        list_personality.append(type_labelized)
        list_posts.append(temp)

    list_posts = np.array(list_posts)
    list_personality = np.array(list_personality)
    return list_posts, list_personality
