# SGDClassifer parameter tunning

import numpy as np
import pandas as pd
import scipy
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.linear_model import SGDClassifier
from sklearn.metrics import accuracy_score, make_scorer, confusion_matrix
from sklearn.model_selection import train_test_split, RandomizedSearchCV, KFold
from sklearn.pipeline import Pipeline
from sklearn.metrics import accuracy_score
from preprocessor import pre_process_data
from preprocessor import translate_personality
from preprocessor import translate_back
#from preprocessor import get_types


SAVE_MODEL = True
MODELS_DIR = "models"
DATA_DIR = "data"
DIMENSIONS = ["IE", "NS", "FT", "PJ"]

data = pd.read_csv('data/mbti_personality.csv');
#data = data.join(data.apply(lambda row: get_types(row), axis=1))

list_posts, list_personality = pre_process_data(data, remove_stop_words=True)

# Learn the vocabulary dictionary and return term-document matrix
Tfidf = TfidfVectorizer(analyzer="word",
                             max_features=1500,
                             tokenizer=None,
                             preprocessor=None,
                             stop_words=None,
                             max_df=0.7,
                             min_df=0.1)



type_indicators = ["IE: Introversion (I) - Extroversion (E)", "NS: Intuition (N) â Sensing (S)",
                   "FT: Feeling (F) - Thinking (T)", "JP: Judging (J) â Perceiving (P)"]
X = list_posts

# Let's train type indicator individually
for l in range(len(DIMENSIONS)):
    print("%s ..." % (type_indicators[l]))

    # Let's train type indicator individually
    Y = list_personality[:, l]

    np.random.shuffle(X)
    np.random.shuffle(Y)

    # split data into train and test sets
    seed = 7
    test_size = 0.33
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=test_size, random_state=seed)

   # sgd = SGDClassifier(max_iter=10, tol=None)
    sgd = SGDClassifier()

    pipe = Pipeline([("Tfidf", Tfidf), ("sgd", sgd)])

    # Define our search space for Randomized Search
    search_space = [
        {
            'sgd__loss': ['hinge', 'modified_huber'],
            'sgd__penalty': ['none', 'l2', 'l1'],
            'sgd__alpha': scipy.stats.uniform(0.00005, 0.002)

        }
    ]


    # Define cross validation
    kfold = KFold(n_splits=10, random_state=42, shuffle=True)  # put 10 try - yeah tried it

    # AUC and accuracy as score
    scoring = {'AUC': 'roc_auc', 'Accuracy': make_scorer(accuracy_score)}

    # Define grid search
    grid = RandomizedSearchCV(
        pipe,
        param_distributions=search_space,
        cv=kfold,
        scoring=scoring,
        refit='AUC',
        verbose=1,
        n_jobs=-1
    )

    # for param in grid.get_params().keys():
    #     print(param)
    # Fit grid search
    model = grid.fit(X_train, y_train)

    # The model scores and confusion matrix can be obtained by
    predict = model.predict(X_test)
    print('Best AUC Score: {}'.format(model.best_score_))
    print('Accuracy: {}'.format(accuracy_score(y_test, predict)))
    print(confusion_matrix(y_test, predict))

    # And the best parameters can be obtained by:
    print(model.best_params_)







