# XGBoost parameter training

import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics import accuracy_score, make_scorer, confusion_matrix
from sklearn.model_selection import train_test_split, RepeatedStratifiedKFold, GridSearchCV, StratifiedKFold,  RandomizedSearchCV, KFold
from sklearn.pipeline import Pipeline
from xgboost import XGBClassifier
from preprocessor import pre_process_data
#from preprocessor import get_types

SAVE_MODEL = True
MODELS_DIR = "models"
DATA_DIR = "data"
DIMENSIONS = ["IE", "NS", "FT", "PJ"]

data = pd.read_csv('data/mbti_personality.csv');
#data = data.join(data.apply(lambda row: get_types(row), axis=1))


type_indicators = ["IE: Introversion (I) - Extroversion (E)", "NS: Intuition (N) â Sensing (S)",
                   "FT: Feeling (F) - Thinking (T)", "JP: Judging (J) â Perceiving (P)"]


list_posts, list_personality = pre_process_data(data, remove_stop_words=True)


# Posts in tf-idf representation
Tfidf = TfidfVectorizer(analyzer="word",
                             max_features=1500,
                             tokenizer=None,
                             preprocessor=None,
                             stop_words=None,
                             max_df=0.7,
                             min_df=0.1)




X = list_posts

# setup parameters for xgboost
param = {}
param['n_estimators'] = 200
param['max_depth'] = 2
param['nthread'] = 8
param['learning_rate'] = 0.2

# Let's train type indicator individually
for l in range(len(DIMENSIONS)):
    print("%s ..." % (type_indicators[l]))

    # Let's train type indicator individually
    Y = list_personality[:, l]


    #split data into train and test sets
    seed = 7
    test_size = 0.33
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=test_size, random_state=seed)

    xgb = XGBClassifier(use_label_encoder=False)
    pipe = Pipeline([("Tfidf", Tfidf), ("xgb", xgb)])

    # Define our search space for randomized search
    search_space = [
        {
            'xgb__n_estimators': [50, 100, 150, 200],
            'xgb__learning_rate': [0.01, 0.1, 0.2, 0.3],
            'xgb__max_depth': range(3, 10),
            'xgb__colsample_bytree': [i / 10.0 for i in range(1, 3)],
            'xgb__gamma': [i / 10.0 for i in range(3)]

        }
    ]

    # Define cross validation
    kfold = KFold(n_splits=10, random_state=42, shuffle=True ) # put 10 try and  put shuffle = true,

    # AUC and accuracy as score
    scoring = {'AUC': 'roc_auc', 'Accuracy': make_scorer(accuracy_score)}

    # Define grid search
    grid = RandomizedSearchCV(
        pipe,
        param_distributions=search_space,
        cv=kfold,
        scoring=scoring,
        refit='AUC',
        verbose=1,
        n_jobs=-1
    )

    # for param in grid.get_params().keys():
    #     print(param)
    # Fit grid search
    model = grid.fit(X_train, y_train)

    # The model scores and confusion matrix can be obtained by
    predict = model.predict(X_test)
    print('Best AUC Score: {}'.format(model.best_score_))
    print('Accuracy: {}'.format(accuracy_score(y_test, predict)))
    print(confusion_matrix(y_test, predict))

    # And the best parameters can be obtained by:
    print(model.best_params_)



